package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.*;
import pps.core.base.entity.BaseLineAllView;
import pps.core.base.entity.BaseLineEnt;
import pps.core.base.entity.BaseLineView;
import pps.core.base.mapper.BaseLineMapper;
import pps.core.base.mapper.BaseLineViewMapper;
import pps.core.base.service.data.base_line.*;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@XService
public class BaseLineService {
    @XApiAnonymous
    @XApiPost
    public XServiceResult createBaseLine(XContext context, CreateBaseLineInput input){
        BaseLineMapper mapper = context.getBean(BaseLineMapper.class);
        BaseLineEnt entity = new BaseLineEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBaseLine(XContext context, UpdateBaseLineInput input){
        BaseLineMapper mapper = context.getBean(BaseLineMapper.class);
        QueryWrapper<BaseLineEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseLineEnt::getId, input.getId());
        BaseLineEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBaseLine(XContext context, DeleteBaseLineInput input){
        BaseLineMapper mapper = context.getBean(BaseLineMapper.class);
        QueryWrapper<BaseLineEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseLineEnt::getId, input.getId());
        BaseLineEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XSingleResult<GetBaseLineOutput> getBaseLine(XContext context, GetBaseLineInput input){
        BaseLineMapper mapper = context.getBean(BaseLineMapper.class);
        QueryWrapper<BaseLineEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseLineEnt::getId, input.getId());
        BaseLineEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseLineOutput output = new GetBaseLineOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiPost
    public XPageResult<QueryBaseLineOutput> queryBaseLineList(XContext context, QueryBaseLineListInput input){
        PageHelper.startPage(input.getPage(), input.getLimit());
        //获取组织机构下的子集
        SysOrganizationCloudService cloudService = context.getBean(SysOrganizationCloudService.class);
        GetAllOuListByOuIdInput ouIdInput = new GetAllOuListByOuIdInput();
        if(XStringUtils.isNotEmpty(input.getOuId()))
            ouIdInput.setOuId(input.getOuId());
        else
            ouIdInput.setOuId("00000000-0000-0000-0000-000000000000");
        XListResult<GetOuListTreeOutput> outputXListResult = cloudService.getAllOuListByOuId(context , ouIdInput);
        outputXListResult.throwIfFail();
        List<GetOuListTreeOutput> ouList = outputXListResult.getResult();
        List<String> ouIdList = new ArrayList<>();
        if(CollectionUtils.isEmpty(ouList)){
            ouIdList.add(input.getOuId());
        }else {
            ouIdList = ouList.stream().map(GetOuListTreeOutput::getOuId).collect(Collectors.toList());
        }
        BaseLineMapper mapper = context.getBean(BaseLineMapper.class);
        QueryWrapper<BaseLineEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().in(BaseLineEnt::getWellOuId , ouIdList);
        List<BaseLineEnt> list = mapper.selectList(queryWrapper);
        PageInfo<BaseLineEnt> pageInfo = new PageInfo<>(list);
        List<QueryBaseLineOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseLineOutput.class);
        //获取线路、电网字典
        List<QuerySysDictionaryViewOutput> gridTypeList = getDictList(context , "grid_type");
        List<QuerySysDictionaryViewOutput> lineTypeList = getDictList(context , "line_type");
        //获取上级线路
        List<Integer> parenIds = list.stream().filter(item->item.getParentId() != null).map(BaseLineEnt::getParentId).collect(Collectors.toList());
        List<BaseLineEnt> parentList = new ArrayList<>();
        if(!CollectionUtils.isEmpty(parenIds)){
            queryWrapper.clear();
            queryWrapper.lambda().in(BaseLineEnt::getId , parenIds);
            parentList = mapper.selectList(queryWrapper);
        }
        for(QueryBaseLineOutput item : outputs){
            item.setGridTypeName(getDictName(item.getGridType(), gridTypeList));
            item.setLineTypeName(getDictName(item.getLineType(), lineTypeList));
            GetOuListTreeOutput treeOutput = ouList.stream().filter(tree->tree.getOuId().equals(item.getWellOuId())).findFirst().orElse(null);
            if(null != treeOutput){
                //设置井场
                item.setWellOuName(treeOutput.getOuName());
                //根据井场，设置油田
                GetOuListTreeOutput oilTreeOutput = ouList.stream().filter(tree->tree.getOuId().equals(treeOutput.getParentOuId())).findFirst().orElse(null);
                if(null != oilTreeOutput){
                    item.setOilOuName(oilTreeOutput.getOuName());
                }
            }
            if(item.getParentId() != null){
                //设置父级线路
                BaseLineEnt parent = parentList.stream().filter(p->p.getId() == item.getParentId()).findFirst().orElse(null);
                if(parent != null)
                    item.setPLineName(parent.getLineName());
            }
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XApiAnonymous
    @XApiPost
    public XPageResult<QueryBaseLineOutput> queryBaseLine(XContext context, QueryBaseLineInput input){
        BaseLineViewMapper mapper = context.getBean(BaseLineViewMapper.class);
        BaseLineView view = XCopyUtils.copyNewObject(input,BaseLineView.class);
        List<BaseLineView> list = mapper.selectList(view);
        PageInfo<BaseLineView> pageInfo = new PageInfo(list);
        List<QueryBaseLineOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseLineOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XApiAnonymous
    @XApiPost
    public XPageResult<QueryBaseLineAllOutput> queryAllBaseLine(XContext context, QueryBaseLineInput input){
        BaseLineViewMapper mapper = context.getBean(BaseLineViewMapper.class);
        BaseLineView view = XCopyUtils.copyNewObject(input,BaseLineView.class);
        List<BaseLineAllView> list = mapper.selectDataList(view);
        PageInfo<BaseLineAllView> pageInfo = new PageInfo(list);
        List<QueryBaseLineAllOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseLineAllOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XApiAnonymous
    @XApiPost
    public XPageResult<QueryBaseLineOutput> dynamicQueryBaseLine(XContext context, CustomQueryInput input){
        BaseLineMapper mapper = context.getBean(BaseLineMapper.class);
        return XMapperHelper.query(mapper, input, BaseLineEnt.class, QueryBaseLineOutput.class);
    }

    public List<QuerySysDictionaryViewOutput> getDictList(XContext context , String alias ){
        SystemDictionaryService dictionaryService = context.getBean(SystemDictionaryService.class);
        QuerySysDictionaryViewInput dictInput = new QuerySysDictionaryViewInput();
        dictInput.setAlias(alias);
        dictInput.setPage(1);
        dictInput.setLimit(999);
        XListResult<QuerySysDictionaryViewOutput> dictPageResult = dictionaryService.queryChildSysDictionarysByParentAlias(context, dictInput);
        dictPageResult.throwIfFail();
        return dictPageResult.getResult();
    }

    public String getDictName(String dictKey,List<QuerySysDictionaryViewOutput> dictList){
        if(StringUtils.isBlank(dictKey) || CollectionUtils.isEmpty(dictList)){
            return " ";
        }
        for (QuerySysDictionaryViewOutput dict : dictList) {
            if(dict.getDicKey().equals(dictKey)){
                return dict.getDicName();
            }
        }
        return "";
    }
}
